#include "touch.h"
#include "sbit.h"
#include <util/atomic.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define REPEAT_MASK     (KEY_LEFT | KEY_RIGHT)
#define REPEAT_START    50                              // after 500ms
#define REPEAT_NEXT     20                              // every 200ms

uint8_t key_state;                                      // debounced key state:
uint8_t key_press;                                      // key press detect
uint8_t key_rpt;                                        // key long press and repeat
touch_t touch[2];

void touch_init(void)
{
  DIDR0 = 1<<TOUCHMUX1 | 1<<TOUCHMUX0;
  ADMUX = ADMUX_GND;
  ADCSRA = ADCSRA_START;
  sei();
  _delay_ms(20);
  cli();
  for (uint8_t i = 0; i < sizeof(touch)/sizeof(touch_t); i++)
    touch[i].comp = touch[i].val + THRESHOLD;
}

ISR(ADC_vect, ISR_NOBLOCK)
{
  static uint8_t cnt_10ms;
  static uint8_t key_val;
  uint16_t val = ADC;
  switch (cnt_10ms++)
  {
    case 0:
      touch[1].val = val;
      if (val > touch[1].comp)
        key_val |= KEY_RIGHT;
      TOUCH_PORT1 = 1;                                  // charge next Touch Input
      TOUCH_DDR1 = 1;
      TOUCH_DDR0 = 0;
      TOUCH_PORT0 = 0;                                  // open Touch Input
      ADMUX = 1<<REFS0 | TOUCHMUX0;                     // S&H
      break;
    case 1:
      touch[0].val = val;
      if (val > touch[0].comp)
        key_val |= KEY_LEFT;
      break;
    case (uint64_t)(.5 + ADC_SAMPLE_FREQ(ADCSRA_START) * 10e-3):
      cnt_10ms = 0;
      static uint8_t ct0, ct1, rpt;
      key_val ^= key_state;                             // key changed ?
      ct0 = ~( ct0 & key_val );                         // reset or count ct0
      ct1 = ct0 ^ (ct1 & key_val);                      // reset or count ct1
      key_val &= ct0 & ct1;                             // count until roll over ?
      key_state ^= key_val;                             // then toggle debounced state
      key_press |= key_state & key_val;                 // 0->1: key press detect
      if( (key_state & REPEAT_MASK) == 0 )              // check repeat function
         rpt = REPEAT_START;                            // start delay
      if( --rpt == 0 ){
        rpt = REPEAT_NEXT;                              // repeat delay
        key_rpt |= key_state & REPEAT_MASK;
      }
      key_val = 0;
      TOUCH_PORT0 = 1;                                  // charge next Touch Input
      TOUCH_DDR0 = 1;
      TOUCH_DDR1 = 0;
      TOUCH_PORT1 = 0;                                  // open Touch Input
      ADMUX = 1<<REFS0 | TOUCHMUX1;                     // S&H
  }
  ADCSRA = ADCSRA_START;
  ADMUX = ADMUX_GND;                                    // S&H GND after conversion complete
}

uint8_t get_key_press(uint8_t key_mask)
{
  ATOMIC_BLOCK(ATOMIC_FORCEON)                          // read and clear atomic
  {
    key_mask &= key_press;                              // read key(s)
    key_press ^= key_mask;                              // clear key(s)
  }
  return key_mask;
}

uint8_t get_key_rpt(uint8_t key_mask)
{
  ATOMIC_BLOCK(ATOMIC_FORCEON)                          // read and clear atomic
  {
    key_mask &= key_rpt;                                // read key(s)
    key_rpt ^= key_mask;                                // clear key(s)
  }
  return key_mask;
}

uint8_t get_key_short(uint8_t key_mask)
{
  cli();                                                // read key state and key press atomic
  return get_key_press(~key_state & key_mask);
}

uint8_t get_key_long(uint8_t key_mask)
{
  return get_key_press(get_key_rpt( key_mask));
}
